#region Copyright (c) 2003-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2003-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ComponentGo.Calendars {

#if !DAILY

	/// <summary>
	/// Main calendar control of a library displayind various calendars.
	/// </summary>
	/// <remarks>
	/// Control displays one selected view of a calendar. 
	/// A selection of a calendar view can be done using a <see cref="CurrentView"/> property.
	/// Available calendars can be determined using a <see cref="Views"/> property.<para/>
	/// Selecting a displayed calendar is also possible from an attached toolbar. 
	/// Its display is controlled by a <see cref="ToolbarVisible"/> property.<para/>
	/// All calendar controls use a common collection of appointment 
	/// <see cref="RootCalendar.Appointments"/> objects and a common collection 
	/// of resources <see cref="RootCalendar.Resources"/>.
	/// Also, access to individual calendar in a control is possible using the following properties:
	/// <see cref="DailyCalendar"/>, <see cref="MonthlyCalendar"/>, 
	/// <see cref="ResourceScheduler"/>, <see cref="WeeklyCalendar"/>. 
	/// It is also possible to access a currently selected calendar. <para/>
	/// ComboCalendar can cooperate with a system calendar control 
	/// <see cref="System.Windows.Forms.MonthCalendar"/>  
	/// that after linking with a  <see cref="LinkedCalendar"/> property 
	/// allows for fast switching between date and calendar views.
	/// </remarks>
	[Designer(typeof(Design.BaseCalendarDesigner), typeof(System.ComponentModel.Design.IDesigner))]
	[ToolboxItem(true)]
	[System.Runtime.InteropServices.ComVisible(true)]
	[System.Runtime.InteropServices.Guid("182C9E83-5B0B-4029-86EF-9146883F77BA")]
	public class ComboCalendar : RootCalendar {

		#region General

		private System.Windows.Forms.ToolBar toolBar;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ToolBarButton buttonScheduler;
		private System.Windows.Forms.ToolBarButton buttonDaily;
		private System.Windows.Forms.ToolBarButton buttonWeekly;
		private System.Windows.Forms.ToolBarButton buttonMonthly;
		private System.Windows.Forms.ToolBarButton buttonWorkWeek;
		private System.Windows.Forms.ToolBarButton buttonToday;
		private System.Windows.Forms.ToolBarButton buttonDivider1;
		private System.Windows.Forms.ImageList imageList;

		/// <summary>
		/// Constructor creating a combo calendar control. 
		/// Properties are initialized to standard values.
		/// </summary>
		public ComboCalendar() : base(null) {
			SetStyle(ControlStyles.AllPaintingInWmPaint, false);
			InitializeComponent();
			Controls.Add(toolBar);
			Size = new Size(150, 200);
		}

		#region InitializeComponent
		
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ComboCalendar));
			this.toolBar = new System.Windows.Forms.ToolBar();
			this.buttonToday = new System.Windows.Forms.ToolBarButton();
			this.buttonDivider1 = new System.Windows.Forms.ToolBarButton();
			this.buttonScheduler = new System.Windows.Forms.ToolBarButton();
			this.buttonDaily = new System.Windows.Forms.ToolBarButton();
			this.buttonWorkWeek = new System.Windows.Forms.ToolBarButton();
			this.buttonWeekly = new System.Windows.Forms.ToolBarButton();
			this.buttonMonthly = new System.Windows.Forms.ToolBarButton();
			this.imageList = new System.Windows.Forms.ImageList(this.components);
			// 
			// toolBar
			// 
			this.toolBar.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.toolBar.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																					   this.buttonToday,
																					   this.buttonDivider1,
																					   this.buttonScheduler,
																					   this.buttonDaily,
																					   this.buttonWorkWeek,
																					   this.buttonWeekly,
																					   this.buttonMonthly});
			this.toolBar.Divider = false;
			this.toolBar.DropDownArrows = true;
			this.toolBar.ImageList = this.imageList;
			this.toolBar.Location = new System.Drawing.Point(17, 17);
			this.toolBar.Name = "toolBar";
			this.toolBar.ShowToolTips = true;
			this.toolBar.Size = new System.Drawing.Size(0, 151);
			this.toolBar.TabIndex = 0;
			this.toolBar.TextAlign = System.Windows.Forms.ToolBarTextAlign.Right;
			this.toolBar.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBar_ButtonClick);
			// 
			// buttonToday
			// 
			this.buttonToday.Text = (string)resources.GetObject("buttonToday.Text");
			this.buttonToday.ToolTipText = (string)resources.GetObject("buttonToday.ToolTipText");
			// 
			// buttonDivider1
			// 
			this.buttonDivider1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// buttonScheduler
			// 
			this.buttonScheduler.ImageIndex = 0;
			this.buttonScheduler.Text = (string)resources.GetObject("buttonScheduler.Text");
			this.buttonScheduler.ToolTipText = (string)resources.GetObject("buttonScheduler.ToolTipText");
			// 
			// buttonDaily
			// 
			this.buttonDaily.ImageIndex = 1;
			this.buttonDaily.Text = (string)resources.GetObject("buttonDaily.Text");
			this.buttonDaily.ToolTipText = (string)resources.GetObject("buttonDaily.ToolTipText");
			this.buttonDaily.Pushed = true;
			// 
			// buttonWorkWeek
			// 
			this.buttonWorkWeek.ImageIndex = 2;
			this.buttonWorkWeek.Text = (string)resources.GetObject("buttonWorkWeek.Text");
			this.buttonWorkWeek.ToolTipText = (string)resources.GetObject("buttonWorkWeek.ToolTipText");
			// 
			// buttonWeekly
			// 
			this.buttonWeekly.ImageIndex = 3;
			this.buttonWeekly.Text = (string)resources.GetObject("buttonWeekly.Text");
			this.buttonWeekly.ToolTipText = (string)resources.GetObject("buttonWeekly.ToolTipText");
			// 
			// buttonMonthly
			// 
			this.buttonMonthly.ImageIndex = 4;
			this.buttonMonthly.Text = (string)resources.GetObject("buttonMonthly.Text");
			this.buttonMonthly.ToolTipText = (string)resources.GetObject("buttonMonthly.ToolTipText");
			// 
			// imageList
			// 
			this.imageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imageList.ImageSize = new System.Drawing.Size(16, 15);
			this.imageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList.ImageStream")));
			this.imageList.TransparentColor = System.Drawing.Color.Transparent;

		}

		#endregion

		/// <summary>
		/// Overloaded. Implementation of a calendar.
		/// </summary>
		/// <param name="disposing">Like overloaded</param>
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components!=null)
					components.Dispose();
			}
			base.Dispose(disposing);
		}

		#endregion

		#region Hide properties

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new Control.ControlCollection Controls {
			get { return base.Controls; }
		}

		#endregion

		#region Calendars

		const CalendarView AllViews = CalendarView.Scheduler | CalendarView.Daily | CalendarView.Weekly | CalendarView.Monthly | CalendarView.WorkWeek;
		static readonly CalendarView[] EnumViews = new CalendarView[] {CalendarView.Daily, CalendarView.WorkWeek, CalendarView.Weekly, CalendarView.Monthly, CalendarView.Scheduler};

		CalendarView views = AllViews;

		/// <summary>
		/// Determines calendars that can be displayed by a combo calendar.
		/// </summary>
		/// <remarks>
		/// If a given calendar cannot be displayed a value of property corresponding 
		/// to a calendar control of a given type will be <b>null</b>.
		/// </remarks>
		[Category("Calendars")]
		[Description("Determines calendars that can be displayed by a combo calendar.")]
		[DefaultValue(AllViews)]
		[Editor(typeof(Forms.Design.FlagsEditor), typeof(System.Drawing.Design.UITypeEditor))]
		public CalendarView Views {
			get { return views; }
			set {
				if (value==0) throw new ArgumentException();
				views = value;
				if (!IsAllowedView(CurrentView)) {
					foreach (CalendarView view in EnumViews) {
						if (IsAllowedView(view)) {
							CurrentView = view;
							break;
						}
					}
				}
				foreach (CalendarView view in EnumViews)
					getButton(view).Visible = IsAllowedView(view);

				if (!IsAllowedView(CalendarView.Daily) && !IsAllowedView(CalendarView.WorkWeek) && dailyCal!=null) {
					dailyCal.Dispose();
					dailyCal = null;
				}
				if (!IsAllowedView(CalendarView.Weekly) && weeklyCal!=null) {
					weeklyCal.Dispose();
					weeklyCal = null;
				}
				if (!IsAllowedView(CalendarView.Monthly) && monthlyCal!=null) {
					monthlyCal.Dispose();
					monthlyCal = null;
				}
				if (!IsAllowedView(CalendarView.Scheduler) && schedulerCal!=null) {
					schedulerCal.Dispose();
					schedulerCal = null;
				}
				calendars = null;
				OnViewsChanged(EventArgs.Empty);
			}
		}

		/// <summary>
		/// An event raized after the <see cref="Views"/> property change.
		/// </summary>
		[Category("Property Changed")]
		[Description("An event raized after the Views property change")]
		public event EventHandler ViewsChanged;

		/// <summary>
		///	Raises the <see cref="ViewsChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnViewsChanged(EventArgs args) {
			if (ViewsChanged!=null)
				ViewsChanged(this, args);
		}

		CalendarView currentView = CalendarView.Daily;

		/// <summary>
		/// Determines a calendar currently displayed by a control.
		/// </summary>
		/// <remarks>
		/// Changing value of this property results in changing a calendar displayed by a combo calendar control. 
		/// You can change a displayed calendar clicking on a toolbar.
		/// </remarks>
		[Category("Calendars")]
		[Description("Determines a calendar currently displayed by a control.")]
		[DefaultValue(CalendarView.Daily)]
		public CalendarView CurrentView {
			get { return currentView; }
			set {
				if (CurrentView==value) return;
				currentView = value;
				BaseCalendar current = CurrentCalendar;
				foreach (BaseCalendar bc in Calendars) {
					bool v = bc==current;
					if (bc.Visible!=v) 
						bc.Visible = v;
				}
				resizeCalendar();

				Appointments.calendar = current;
				Resources.calendar = current;
				current.Resync(true);

				foreach (CalendarView view in EnumViews)
					getButton(view).Pushed = value==view;

				switch (value) {
					case CalendarView.Daily:
						if (dailyCal.VisibleColumns==dailyCal.ResCount*5 && IsAllowedView(CalendarView.WorkWeek))
							dailyCal.VisibleColumns = dailyCal.ResCount;
						break;
					case CalendarView.WorkWeek:
						dailyCal.VisibleColumns = dailyCal.ResCount*5;
						break;
				}

				OnCurrentViewChanged(EventArgs.Empty);
				syncLinkedCalendar();
			}
		}

		/// <summary>
		/// Determines whether it is possible to select a given calendar view.
		/// <seealso cref="Views"/>
		/// </summary>
		/// <param name="value">A calendar view to be checked.</param>
		/// <returns>Returns <b>true</b> when a given calendar can be displayed 
		/// and <b>false</b> when it cannot.</returns>
		public bool IsAllowedView(CalendarView value) {
			return (views & value)!=0;
		}

		void addCalendar(BaseCalendar cal, CalendarView view) {
			cal.Visible = false;
			Controls.Add(cal);
			calendars = null;
			resizeCalendar();
			cal.AllowOverlap = AllowOverlap;
			cal.AllowDrop = AllowDrop;
			cal.GrowMode = GrowMode;
			cal.EnableEditInPlace = EnableEditInPlace;
			if (Appointments.calendar==null)
				Appointments.calendar = cal;
			OnCalendarCreated(new CalendarCreatedEventArgs(cal, view));
		}

		/// <summary>
		/// Called immediately after creating a new calendar control inside a ComboCalendar.
		/// </summary>
		/// <remarks>
		/// This event is a good place to programmaticalyy initialize a calendar control. 
		/// It is called for every calendar (View) independently.
		/// You can learn what calendar has been created examining properties 
		/// of a <see cref="CalendarCreatedEventArgs"/> object.
		/// </remarks>
		[Category("Behavior")]
		[Description("Called immediately after creating a new calendar control inside a ComboCalendar")]
		public event CalendarCreatedEventHandler CalendarCreated;

		/// <summary>
		///	Raises the <see cref="CalendarCreated"/> event.
		/// </summary>
		/// <param name="args">An <see cref="CalendarCreatedEventArgs"/> that contains the event data.</param>
		protected virtual void OnCalendarCreated(CalendarCreatedEventArgs args) {
			if (CalendarCreated!=null)
				CalendarCreated(this, args);
		}

		void resizeCalendar() {
			int theight = ToolbarVisible ? toolBar.Height : 0;
			CurrentCalendar.SetBounds(0, theight, Width, Height-theight, BoundsSpecified.All);
		}

		/// <summary>
		/// An event raized after the <see cref="CurrentView"/> property change.
		/// </summary>
		[Category("Property Changed")]
		public event EventHandler CurrentViewChanged;

		/// <summary>
		///	Raises the <see cref="CurrentViewChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnCurrentViewChanged(EventArgs args) {
			if (CurrentViewChanged!=null)
				CurrentViewChanged(this, args);
		}

		/// <summary>
		/// Allows to read a calendar control currently displayed inside a calendar <see cref="ComboCalendar"/>.
		/// </summary>
		/// <remarks>
		/// Depending on a value of <see cref="CurrentView"/> control returns one of the following controls:
		/// <see cref="DailyCalendar"/>, <see cref="WeeklyCalendar"/>, 
		/// <see cref="MonthlyCalendar"/>, <see cref="ResourceScheduler"/>.
		/// </remarks>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public BaseCalendar CurrentCalendar {
			get {
				switch (CurrentView) {
					case CalendarView.Daily: 
					case CalendarView.WorkWeek: return DailyCalendar;
					case CalendarView.Weekly: return WeeklyCalendar;
					case CalendarView.Monthly: return MonthlyCalendar;
					case CalendarView.Scheduler: return ResourceScheduler;
				}
				throw new InvalidOperationException();
			}
		}

		ResourceScheduler schedulerCal = null;

		/// <summary>
		/// Calendar control displayed in a sheduler mode.
		/// </summary>
		/// <remarks>
		/// Allows to assign settings to a calendar in <see cref="CalendarView.Scheduler"/> mode.
		/// </remarks>
		[Description("Calendar control displayed in a sheduler mode.")]
		[Category("Calendars")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public ResourceScheduler ResourceScheduler {
			get {
				if (schedulerCal==null && IsAllowedView(CalendarView.Scheduler))
					addCalendar(schedulerCal = new ResourceScheduler(this), CalendarView.Scheduler);
				return schedulerCal;
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="ResourceScheduler"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeResourceScheduler() {
			return schedulerCal!=null;
		}

		DailyCalendar dailyCal = null;

		/// <summary>
		/// A calendar control displayed in a daily and work week modes.
		/// </summary>
		/// <remarks>
		/// Allows to assign settings to a calendar in  mode <see cref="CalendarView.Daily"/> or <see cref="CalendarView.WorkWeek"/>.
		/// </remarks>
		[Description("A calendar control displayed in a daily and work week modes.")]
		[Category("Calendars")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public DailyCalendar DailyCalendar {
			get {
				if (dailyCal==null && (IsAllowedView(CalendarView.Daily) || IsAllowedView(CalendarView.WorkWeek)))
					addCalendar(dailyCal = new DailyCalendar(this), CalendarView.Daily | CalendarView.WorkWeek);
				return dailyCal;
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="DailyCalendar"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeDailyCalendar() {
			return dailyCal!=null;
		}

		WeeklyCalendar weeklyCal = null;

		/// <summary>
		/// Calendar control displayed in a weekly mode.
		/// </summary>
		/// <remarks>
		/// Allows to assign settings to a calendar in  mode <see cref="CalendarView.Weekly"/>.
		/// </remarks>
		[Description("Kontrolka kalendarza wywietlanego w trybie weekly.")]
		[Category("Calendars")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public WeeklyCalendar WeeklyCalendar {
			get {
				if (weeklyCal==null && IsAllowedView(CalendarView.Weekly))
					addCalendar(weeklyCal = new WeeklyCalendar(this), CalendarView.Weekly);
				return weeklyCal;
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="WeeklyCalendar"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeWeeklyCalendar() {
			return weeklyCal!=null;
		}

		MonthlyCalendar monthlyCal = null;

		/// <summary>
		/// Calendar control displayed in a monthly mode.
		/// </summary>
		/// <remarks>
		/// Allows to assign settings to a calendar in  mode <see cref="CalendarView.Monthly"/>.
		/// </remarks>
		[Description("Calendar control displayed in a monthly mode.")]
		[Category("Calendars")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public MonthlyCalendar MonthlyCalendar {
			get {
				if (monthlyCal==null && IsAllowedView(CalendarView.Monthly))
					addCalendar(monthlyCal = new MonthlyCalendar(this), CalendarView.Monthly);
				return monthlyCal;
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="MonthlyCalendar"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeMonthlyCalendar() {
			return monthlyCal!=null;
		}

		BaseCalendar[] calendars = null;

		BaseCalendar[] Calendars {
			get {
				if (calendars==null) {
					int count = 0;
					if (dailyCal!=null) ++count;
					if (monthlyCal!=null) ++count;
					if (weeklyCal!=null) ++count;
					if (schedulerCal!=null) ++count;
					calendars = new BaseCalendar[count];
					int i = 0;
					if (dailyCal!=null) calendars[i++] = dailyCal;
					if (monthlyCal!=null) calendars[i++] = monthlyCal;
					if (weeklyCal!=null) calendars[i++] = weeklyCal;
					if (schedulerCal!=null) calendars[i++] = schedulerCal;
				}
				return calendars;
			}
		}

		#endregion

		#region Toolbar

		/// <summary>
		/// Determines visibility of a toolbar control in the upper part of a Combo control.
		/// </summary>
		/// <remarks>
		/// Toolbar enables a calendar operator top change a currently displayed calendar 
		/// <see cref="CurrentView"/> and give a quick access to today's date in a  calendar.<para/>
		/// A number of buttons displayed in a toolbar is controlled by a <see cref="Views"/> property.
		/// </remarks>
		[Description("Determines visibility of a toolbar control in the upper part of a Combo control.")]
		[DefaultValue(true)]
		[Category("Behavior")]
		public bool ToolbarVisible {
			get { return toolBar.Visible; }
			set { 
				toolBar.Visible = value; 
				OnToolbarVisibleChanged(EventArgs.Empty);
			}
		}

		/// <summary>
		/// An event raized after the <see cref="ToolbarVisible"/> property change.
		/// </summary>
		[Category("Property Changed")]
		[Description("An event raized after the ToolbarVisible property change")]
		public event EventHandler ToolbarVisibleChanged;

		/// <summary>
		///	Raises the <see cref="ToolbarVisibleChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnToolbarVisibleChanged(EventArgs args) {
			if (ToolbarVisibleChanged!=null)
				ToolbarVisibleChanged(this, args);
		}

		private void toolBar_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e) {
			foreach (CalendarView view in EnumViews) {
				if (e.Button==getButton(view)) {
					CurrentView = view;
					return;
				}
			}
			if (e.Button==buttonToday) {
				BaseCalendar bc = CurrentCalendar;
				bc.checkSelected();
				TimeSpan ts = bc.SelectedDateEnd-bc.SelectedDateBegin;
				TimeSpan oneDay = TimeSpan.FromDays(1);
				if (ts>oneDay)
					ts = oneDay;
				DateTime td = DateTime.Today;
				if (ts<oneDay) {
					bc.SelectedDateBegin = td+bc.SelectedDateBegin.TimeOfDay;
					bc.SelectedDateEnd = bc.SelectedDateBegin+ts;
				}
				else {
					bc.SelectedDateBegin = td;
					bc.SelectedDateEnd = td+ts;
				}
				CurrentCalendar.EnsureSelectionVisible();
			}
		}

		ToolBarButton getButton(CalendarView view) {
			switch (view) {
				case CalendarView.Daily: return buttonDaily;
				case CalendarView.Monthly: return buttonMonthly;
				case CalendarView.Scheduler: return buttonScheduler;
				case CalendarView.Weekly: return buttonWeekly;
				case CalendarView.WorkWeek: return buttonWorkWeek;
			}
			throw new ArgumentException();
		}

		#region Colors

		Color toolbarColor = Color.DodgerBlue;

		/// <summary>
		/// A color of a toolbar.
		/// </summary>
		[Category("Appearance")]
		[Description("A color of a toolbar.")]
		public Color ToolbarColor {
			get { return toolbarColor; }
			set {
				if (toolbarColor==value) return;
				if (CalendarTools.Dark(toolbarColor)==toolbarColorSecond)
					toolbarColorSecond = CalendarTools.Dark(value);
				toolbarColor = value;
				toolBar.Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="ToolbarColor"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeToolbarColor() {
			return toolbarColor!=Color.DodgerBlue;
		}

		Color toolbarColorSecond = CalendarTools.Dark(Color.DodgerBlue);

		/// <summary>
		/// Second color for a <see cref="ToolbarColor"/> property used 
		/// for a linear gradient
		/// </summary>
		[Category("Appearance")]
		[Description("Second color for a ToolbarColor property used for a linear gradient")]
		public Color ToolbarColorSecond {
			get { return toolbarColorSecond; }
			set {
				if (toolbarColorSecond==value) return;
				toolbarColorSecond = value;
				toolBar.Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="ToolbarColorSecond"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeToolbarColorSecond() {
			return toolbarColorSecond!=CalendarTools.Dark(toolbarColor);
		}

		/// <summary>
		/// Sets a default value of a property <see cref="ToolbarColorSecond"/>.
		/// </summary>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="ComboCalendar"/>.
		/// </remarks>
		public void ResetWorkHourColorSecond() {
			ToolbarColorSecond = CalendarTools.Dark(toolbarColor);
		}

		#endregion

		#endregion

		#region Overrides

		void checkControls() {
			if (Visible) {
				CurrentCalendar.Visible = true;
				resizeCalendar();
			}
		}

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		protected override void OnCreateControl() {
			base.OnCreateControl();
			checkControls();
		}

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		/// <param name="e">Like overloaded</param>
		protected override void OnVisibleChanged(EventArgs e) {
			base.OnVisibleChanged(e);
			if (Visible)
				checkControls();
		}

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		/// <param name="e">Like overloaded</param>
		protected override void OnResize(EventArgs e) {
			base.OnResize(e);
			resizeCalendar();
		}

		#endregion

		#region Paint

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		/// <param name="pevent">Like overloaded</param>
		protected override void OnPaintBackground(PaintEventArgs pevent) {
			if (ToolbarVisible && Width!=0) {
				CalendarTools.FillRectangle(pevent.Graphics, 
					new Rectangle(0, 0, Width, toolBar.Height),
					ToolbarColor, ToolbarColorSecond, true);
			}
		}

		#endregion

		#region Selected Dates

		/// <summary>
		/// Date and hour of a beginning of a selected time period.
		/// </summary>
		/// <remarks>
		/// Selected time period is painted on a calendar with a special color:
		/// <see cref="BaseCalendar.SelectedColor"/>.<para/>
		/// Setting this property results in a <see cref="RootCalendar.SelectionChanged"/> event 
		/// being generated.
		/// </remarks>
		/// <example>
		/// The example shows how to select a whole today's day:
		/// <code>
		/// calendar.SelectedDateBegin = DateTime.Today;
		/// calendar.SelectedDateEnd = DateTime.Today.AddDays(1);
		/// </code>
		/// <seealso cref="SelectedDateEnd"/>
		/// </example>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public DateTime SelectedDateBegin {
			get { return CurrentCalendar.SelectedDateBegin; }
			set {
				CurrentCalendar.SelectedDateBegin = value;
			}
		}

		/// <summary>
		/// Date and hour of the end of a selected time period
		/// </summary>
		/// <remarks>
		/// Selected time period is painted on a calendar with a special color:
		/// <see cref="BaseCalendar.SelectedColor"/>.<para/>
		/// Setting this property results in a <see cref="RootCalendar.SelectionChanged"/> event 
		/// being generated.
		/// </remarks>
		/// <example>
		/// The example shows how to select a whole today's day:
		/// <code>
		/// calendar.SelectedDateBegin = DateTime.Today;
		/// calendar.SelectedDateEnd = DateTime.Today.AddDays(1);
		/// </code>
		/// <seealso cref="SelectedDateBegin"/>
		/// </example>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public DateTime SelectedDateEnd {
			get { return CurrentCalendar.SelectedDateEnd; }
			set {
				CurrentCalendar.SelectedDateEnd = value;
			}
		}

		/// <summary>
		/// Index of a selected <see cref="Resource"/> object.
		/// </summary>
		/// <remarks>
		/// Selected resource is painted on a calendar with a special color:
		/// <see cref="BaseCalendar.SelectedColor"/>.<para/>
		/// Setting this property results in a <see cref="RootCalendar.SelectionChanged"/> event 
		/// being generated.
		/// </remarks>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public int SelectedResource {
			get { return CurrentCalendar.SelectedResource; }
			set {
				CurrentCalendar.SelectedResource = value;
			}
		}

		/// <summary>
		/// Forces visibility of a given day and an hour in a calendar.
		/// </summary>
		/// <param name="date">Day to be visible.</param>
		public void EnsureVisible(DateTime date) {
			CurrentCalendar.EnsureVisible(date, date, SelectedResource);
		}

		/// <summary>
		/// Forces visibility of a given resource with specified index.
		/// </summary>
		/// <param name="resourceIdx">Index of resource to be visible.</param>
		public void EnsureVisible(int resourceIdx) {
			CurrentCalendar.EnsureVisible(SelectedDateBegin, SelectedDateEnd, resourceIdx);
		}

		/// <summary>
		/// Forces visibility of selected data on a calendar.
		/// </summary>
		public void EnsureSelectionVisible() {
			CurrentCalendar.EnsureVisible(SelectedDateBegin, SelectedDateEnd, SelectedResource);
		}

		#endregion

		#region Visible Dates

		/// <summary>
		/// An date and hour displayed as the first.
		/// </summary>
		[Category("Data")]
		[Description("An date and hour displayed as the first.")]
		public DateTime FirstDateTime {
			get { return CurrentCalendar.VisibleFirstDateTime; }
			set { CurrentCalendar.VisibleFirstDateTime = value; 	}
		}

		/// <summary>
		/// Indicates whether the <see cref="FirstDateTime"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="DailyCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeFirstDateTime() {
			return FirstDateTime!=DateTime.Today;
		}

		/// <summary>
		/// A number of days visible in a calendar.
		/// </summary>
		/// <remarks>
		/// Setting number of days displayed in a calendar can change a current view 
		/// adapting it to a optimum view presenting that number of days.<para/>
		/// First visible day can be set with a <see cref="FirstDateTime"/> property.
		/// </remarks>
		[DefaultValue(1)]
		[Category("Behavior")]
		[Description("A number of days visible in a calendar.")]
		public int VisibleDays {
			get { return (int)CurrentCalendar.VisibleTimeSpan.TotalDays; }
			set {
				bool ok = false;
				if (value==5)  {
					ok = IsAllowedView(CalendarView.WorkWeek);
					if (ok)
						CurrentView = CalendarView.WorkWeek;
				}
				else if (7<=value && value<14) {
					ok = IsAllowedView(CalendarView.Weekly);
					if (ok)
						CurrentView = CalendarView.Weekly;
				}
				else if (value>=14) {
					ok = IsAllowedView(CalendarView.Monthly);
					if (ok)
						CurrentView = CalendarView.Monthly;
				}
				if (!ok) {
					if (!IsAllowedView(CalendarView.Daily))
						throw new ArgumentException();
					CurrentView = CalendarView.Daily;
				}
				CurrentCalendar.VisibleTimeSpan = TimeSpan.FromDays(value);
			}
		}

		System.Windows.Forms.MonthCalendar linkedCalendar = null;

		/// <summary>
		/// A linked external navigation calendar.
		/// </summary>
		/// <remarks>
		/// A navigation calendar is a system calendar control <see cref="System.Windows.Forms.MonthCalendar"/>. 
		/// Allows to change a currently selected period of time. 
		/// A navigation calendar allows also to quickly select a period of time displayed in a calendar. 
		/// Selecting a long enough period results in automatical change of a calendar view.
		/// <seealso cref="VisibleDays"/>
		/// </remarks>
		[DefaultValue(null)]
		[Category("Behavior")]
		public System.Windows.Forms.MonthCalendar LinkedCalendar {
			get { return linkedCalendar; }
			set { 
				if (linkedCalendar==value) return;
				if (linkedCalendar!=null) {
					linkedCalendar.DateChanged -= new DateRangeEventHandler(linkedCalendar_DateSelected);
					linkedCalendar.MouseUp -= new MouseEventHandler(linkedCalendar_MouseUp);
					linkedCalendar.KeyUp -= new KeyEventHandler(linkedCalendar_KeyUp);
				}
				linkedCalendar = value; 
				if (linkedCalendar!=null) {
					linkedCalendar.DateChanged += new DateRangeEventHandler(linkedCalendar_DateSelected);
					linkedCalendar.MouseUp += new MouseEventHandler(linkedCalendar_MouseUp);
					linkedCalendar.KeyUp += new KeyEventHandler(linkedCalendar_KeyUp);
					syncLinkedCalendar();
				}
			}
		}	

		bool innerSyncLink = false;

		private void linkedCalendar_DateSelected(object sender, DateRangeEventArgs e) {
			try {
				innerSyncLink = true;
				FirstDateTime = e.Start;
				VisibleDays = (int)(e.End-e.Start).TotalDays+1;
			}
			finally {
				innerSyncLink = false;
			}
		}

		private void linkedCalendar_MouseUp(object sender, MouseEventArgs e) {
			syncLinkedCalendar();
		}

		private void linkedCalendar_KeyUp(object sender, KeyEventArgs e) {
			syncLinkedCalendar();
		}

		void syncLinkedCalendar() {
			if (linkedCalendar==null || innerSyncLink) return;
			DateTime dt = FirstDateTime;
			linkedCalendar.SelectionRange = new SelectionRange(dt, dt.AddDays(VisibleDays-1));
		}

		/// <summary>
		///	Raises the <see cref="RootCalendar.FirstDateChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected override void OnFirstDateChanged(EventArgs args) {
			base.OnFirstDateChanged (args);
			syncLinkedCalendar();
		}

		#endregion
	}

#endif
}
