Imports System.ComponentModel
Imports DevExpress.XtraEditors

Imports Phoenix.Data
Imports Phoenix.DataAccess
Imports Phoenix.Events

Namespace Forms

    Public Class ChildFormBase : Inherits PhoenixFormBase

#Region " Form Registration "

        Shared Sub New()
            RegisterAvailableForms()
        End Sub


        Private Shared Sub RegisterAvailableForms()
            mFormTypes.Add(GetType(DataSetData.Client), GetType(ClientForm))
            mFormTypes.Add(GetType(DataSetData.Employee), GetType(EmployeeForm))
            mFormTypes.Add(GetType(DataSetData.Product), GetType(ProductForm))
            mFormTypes.Add(GetType(DataSetData.Service), GetType(ServiceForm))
            mFormTypes.Add(GetType(DataSetBill.PettyCashBill), GetType(BillNoSaleForm))

            mFormTypes.Add(GetType(DataSetAppointments.Appointment), GetType(AppointmentForm))
            mFormTypes.Add(GetType(DataSetBill.Bill), GetType(BillForm))
        End Sub

        Private Shared mFormTypes As Hashtable = New Hashtable
        Public Shared ReadOnly Property FormType(ByVal rowType As Type) As Type
            Get
                Dim ft As Type = CType(mFormTypes(rowType), Type)
                Debug.Assert(Not ft Is Nothing, "The requested form is not registered")
                Return ft
            End Get
        End Property

        Private Shared mFormList As Hashtable = New Hashtable
        Private Shared ReadOnly Property FormBag(ByVal frmType As Type) As ArrayList
            Get
                Dim forms As ArrayList = CType(mFormList(frmType), ArrayList)
                If forms Is Nothing Then
                    forms = New ArrayList
                    mFormList.Add(frmType, forms)
                End If
                Return forms
            End Get
        End Property

        Public Shared Shadows Function CloseForm(ByVal row As DataRow) As Boolean
            Dim form As ChildFormBase = FindForm(row)
            If form Is Nothing Then Return True

            form.Close()
            If form.Visible Then
                Return False
            Else
                Return True
            End If
        End Function

        Public Shared Shadows Function FindForm(ByVal row As DataRow) As ChildFormBase
            If row Is Nothing Then Return Nothing
            If row.RowState = DataRowState.Added Then Return Nothing

            For Each form As ChildFormBase In FormBag(FormType(row.GetType()))
                If AreDataRowsSame(form.DataSource, row) Then
                    Return form
                End If
            Next
            Return Nothing
        End Function


        Private Sub RegisterFormInstance(ByVal sender As Object, ByVal args As EventArgs) Handles MyBase.Load
            FormBag(Me.GetType()).Add(Me)
        End Sub

        Private Sub UnregisterFormUnstance(ByVal sender As Object, ByVal args As EventArgs) Handles MyBase.Closed
            FormBag(Me.GetType()).Remove(Me)
        End Sub

#End Region

#Region " Data "
        Public Event DataSourceChanged(ByVal sender As Object, ByVal e As EventArgs)
        Private mDataSource As DataRow
        Public Property DataSource() As DataRow
            Get
                Return mDataSource
            End Get
            Set(ByVal Value As DataRow)
                Debug.Assert(Not Value Is Nothing, "Override CreateNewRow method.")

                DetachDataSource()
                AttachDataSource(Value)

                If Created Then DataBind()
            End Set
        End Property

        Private Sub AttachDataSource(ByVal Row As DataRow)
            mDataSource = Row
            If Not mDataSource Is Nothing Then
                AddHandler mDataSource.Table.RowChanged, AddressOf DataSource_Changed
            End If
            RaiseEvent DataSourceChanged(Me, EventArgs.Empty)
        End Sub

        Private Sub DetachDataSource()
            If Not mDataSource Is Nothing Then
                RemoveHandler mDataSource.Table.RowChanged, AddressOf DataSource_Changed
            End If
            mDataSource = Nothing
        End Sub

        Private Sub DataSource_Changed(ByVal sender As Object, ByVal e As System.Data.DataRowChangeEventArgs)

        End Sub

        Public Overridable ReadOnly Property Title() As String
            Get
                Return DocumentTitle & " - " & DocumentType
            End Get
        End Property

        Public Overridable ReadOnly Property DocumentTitle() As String
            Get
                Return Res.TitleUnknownDocument
            End Get
        End Property

        Public Overridable ReadOnly Property DocumentType() As String
            Get
                Return Res.TypeUnknownDocument
            End Get
        End Property

        Protected Overridable Function IsDataUpdated() As Boolean
            Return DataSource.RowState <> DataRowState.Unchanged
        End Function

        Protected Overridable Sub RejectChanges()
            DataSource.RejectChanges()
        End Sub

#End Region

#Region " Opening Form "
        Public Shared Function OpenForm(ByVal obj As Object) As ChildFormBase
            Dim row As DataRow = CType(obj, DataRow)
            AppCursor.SetBusy()
            Try
                Dim form As ChildFormBase = FindForm(row)

                If Not form Is Nothing Then
                    form.DeleteBarItem.Enabled = True
                    form.Activate()
                Else
                    form = CType(Activator.CreateInstance(FormType(row.GetType())), ChildFormBase)
                    If row Is Nothing Then
                        form.NewRow()
                        form.DeleteBarItem.Enabled = False
                    Else
                        form.DeleteBarItem.Enabled = True
                        form.DataSource = row
                    End If
                    form.Show()
                    form.Activate()
                    '                    form.Focus()
                End If
                Return form
            Finally
                AppCursor.SetFree()
            End Try
        End Function


        Public Shared Sub OpenForm(ByVal type As Type)
            OpenForm(type, Nothing)
        End Sub

        Public Shared Sub OpenForm(ByVal type As Type, ByVal CreationParam As Object)
            Dim form As ChildFormBase = CType(Activator.CreateInstance(FormType(type)), ChildFormBase)
            form.CreationParameter = CreationParam
            form.NewRow()
            form.Show()
        End Sub

#End Region

#Region " Windows Form Designer generated code "

        Public Sub New()
            MyBase.New()

            'This call is required by the Windows Form Designer.
            InitializeComponent()

            'Add any initialization after the InitializeComponent() call
            BarManager.Images = AppSharedComponents.ToolImages
            BarManager.Controller = AppSharedComponents.BarAndDockingController
        End Sub

        'Form overrides dispose to clean up the component list.
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not (components Is Nothing) Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        'Required by the Windows Form Designer
        Private components As System.ComponentModel.IContainer

        'NOTE: The following procedure is required by the Windows Form Designer
        'It can be modified using the Windows Form Designer.  
        'Do not modify it using the code editor.
        Friend WithEvents BarManager As DevExpress.XtraBars.BarManager
        Friend WithEvents MainMenuBar As DevExpress.XtraBars.Bar
        Friend WithEvents FileMenuBarItem As DevExpress.XtraBars.BarSubItem
        Friend WithEvents SaveBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents SaveNewBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents DeleteBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents CloseBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents EditMenuBarItem As DevExpress.XtraBars.BarSubItem
        Friend WithEvents UndoBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents CutBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents CopyBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents PasteBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents ClearBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents SelectBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents ViewMenuBarItem As DevExpress.XtraBars.BarSubItem
        Friend WithEvents ToolbarsBarItem As DevExpress.XtraBars.BarSubItem
        Friend WithEvents ToolbarsListBarItem As DevExpress.XtraBars.BarToolbarsListItem
        Friend WithEvents HelpMenuBarItem As DevExpress.XtraBars.BarSubItem
        Friend WithEvents AboutBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents StandardBar As DevExpress.XtraBars.Bar
        Friend WithEvents barDockControlTop As DevExpress.XtraBars.BarDockControl
        Friend WithEvents barDockControlBottom As DevExpress.XtraBars.BarDockControl
        Friend WithEvents barDockControlLeft As DevExpress.XtraBars.BarDockControl
        Friend WithEvents barDockControlRight As DevExpress.XtraBars.BarDockControl
        Friend WithEvents SaveCloseBarItem As DevExpress.XtraBars.BarButtonItem
        Friend WithEvents ActionsMenuBarItem As DevExpress.XtraBars.BarSubItem
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.BarManager = New DevExpress.XtraBars.BarManager
            Me.MainMenuBar = New DevExpress.XtraBars.Bar
            Me.FileMenuBarItem = New DevExpress.XtraBars.BarSubItem
            Me.SaveBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.SaveNewBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.DeleteBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.CloseBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.EditMenuBarItem = New DevExpress.XtraBars.BarSubItem
            Me.UndoBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.CutBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.CopyBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.PasteBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.ClearBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.SelectBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.ViewMenuBarItem = New DevExpress.XtraBars.BarSubItem
            Me.ToolbarsBarItem = New DevExpress.XtraBars.BarSubItem
            Me.ToolbarsListBarItem = New DevExpress.XtraBars.BarToolbarsListItem
            Me.ActionsMenuBarItem = New DevExpress.XtraBars.BarSubItem
            Me.HelpMenuBarItem = New DevExpress.XtraBars.BarSubItem
            Me.AboutBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.StandardBar = New DevExpress.XtraBars.Bar
            Me.SaveCloseBarItem = New DevExpress.XtraBars.BarButtonItem
            Me.barDockControlTop = New DevExpress.XtraBars.BarDockControl
            Me.barDockControlBottom = New DevExpress.XtraBars.BarDockControl
            Me.barDockControlLeft = New DevExpress.XtraBars.BarDockControl
            Me.barDockControlRight = New DevExpress.XtraBars.BarDockControl
            CType(Me.BarManager, System.ComponentModel.ISupportInitialize).BeginInit()
            Me.SuspendLayout()
            '
            'BarManager
            '
            Me.BarManager.Bars.AddRange(New DevExpress.XtraBars.Bar() {Me.MainMenuBar, Me.StandardBar})
            Me.BarManager.Categories.AddRange(New DevExpress.XtraBars.BarManagerCategory() {New DevExpress.XtraBars.BarManagerCategory("File", New System.Guid("c3aced69-cd2a-4d58-8a80-811fa2689282")), New DevExpress.XtraBars.BarManagerCategory("Edit", New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")), New DevExpress.XtraBars.BarManagerCategory("View", New System.Guid("dfb3a3c7-d8a9-42e3-a350-b3a781047071")), New DevExpress.XtraBars.BarManagerCategory("Help", New System.Guid("99425a3d-94f2-4fca-a6f3-bcd1ea4dd9ef")), New DevExpress.XtraBars.BarManagerCategory("Menu Bar", New System.Guid("e619b5ba-a0ac-4b7d-afa5-b84dd7eab0c0"))})
            Me.BarManager.DockControls.Add(Me.barDockControlTop)
            Me.BarManager.DockControls.Add(Me.barDockControlBottom)
            Me.BarManager.DockControls.Add(Me.barDockControlLeft)
            Me.BarManager.DockControls.Add(Me.barDockControlRight)
            Me.BarManager.Form = Me
            Me.BarManager.Items.AddRange(New DevExpress.XtraBars.BarItem() {Me.FileMenuBarItem, Me.EditMenuBarItem, Me.AboutBarItem, Me.SaveBarItem, Me.SaveCloseBarItem, Me.SaveNewBarItem, Me.DeleteBarItem, Me.CloseBarItem, Me.ViewMenuBarItem, Me.ActionsMenuBarItem, Me.HelpMenuBarItem, Me.UndoBarItem, Me.CutBarItem, Me.CopyBarItem, Me.PasteBarItem, Me.ClearBarItem, Me.SelectBarItem, Me.ToolbarsListBarItem, Me.ToolbarsBarItem})
            Me.BarManager.MainMenu = Me.MainMenuBar
            Me.BarManager.MaxItemId = 25
            '
            'MainMenuBar
            '
            Me.MainMenuBar.BarName = "Main Menu"
            Me.MainMenuBar.DockCol = 0
            Me.MainMenuBar.DockRow = 0
            Me.MainMenuBar.DockStyle = DevExpress.XtraBars.BarDockStyle.Top
            Me.MainMenuBar.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.FileMenuBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.EditMenuBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.ViewMenuBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.ActionsMenuBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.HelpMenuBarItem)})
            Me.MainMenuBar.OptionsBar.MultiLine = True
            Me.MainMenuBar.OptionsBar.UseWholeRow = True
            Me.MainMenuBar.Text = "Main Menu"
            '
            'FileMenuBarItem
            '
            Me.FileMenuBarItem.Caption = "&File"
            Me.FileMenuBarItem.CategoryGuid = New System.Guid("e619b5ba-a0ac-4b7d-afa5-b84dd7eab0c0")
            Me.FileMenuBarItem.Id = 0
            Me.FileMenuBarItem.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.SaveBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.SaveNewBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.DeleteBarItem, True), New DevExpress.XtraBars.LinkPersistInfo(Me.CloseBarItem, True)})
            Me.FileMenuBarItem.Name = "FileMenuBarItem"
            '
            'SaveBarItem
            '
            Me.SaveBarItem.Caption = "&Save"
            Me.SaveBarItem.CategoryGuid = New System.Guid("c3aced69-cd2a-4d58-8a80-811fa2689282")
            Me.SaveBarItem.Id = 5
            Me.SaveBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.S))
            Me.SaveBarItem.Name = "SaveBarItem"
            '
            'SaveNewBarItem
            '
            Me.SaveNewBarItem.Caption = "Sav&e And New"
            Me.SaveNewBarItem.CategoryGuid = New System.Guid("c3aced69-cd2a-4d58-8a80-811fa2689282")
            Me.SaveNewBarItem.Id = 6
            Me.SaveNewBarItem.Name = "SaveNewBarItem"
            '
            'DeleteBarItem
            '
            Me.DeleteBarItem.Caption = "&Delete"
            Me.DeleteBarItem.CategoryGuid = New System.Guid("c3aced69-cd2a-4d58-8a80-811fa2689282")
            Me.DeleteBarItem.Id = 7
            Me.DeleteBarItem.ImageIndex = 2
            Me.DeleteBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.D))
            Me.DeleteBarItem.Name = "DeleteBarItem"
            '
            'CloseBarItem
            '
            Me.CloseBarItem.Caption = "&Close"
            Me.CloseBarItem.CategoryGuid = New System.Guid("c3aced69-cd2a-4d58-8a80-811fa2689282")
            Me.CloseBarItem.Id = 4
            Me.CloseBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Alt Or System.Windows.Forms.Keys.F4))
            Me.CloseBarItem.Name = "CloseBarItem"
            '
            'EditMenuBarItem
            '
            Me.EditMenuBarItem.Caption = "&Edit"
            Me.EditMenuBarItem.CategoryGuid = New System.Guid("e619b5ba-a0ac-4b7d-afa5-b84dd7eab0c0")
            Me.EditMenuBarItem.Id = 2
            Me.EditMenuBarItem.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.UndoBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.CutBarItem, True), New DevExpress.XtraBars.LinkPersistInfo(Me.CopyBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.PasteBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.ClearBarItem, True), New DevExpress.XtraBars.LinkPersistInfo(Me.SelectBarItem)})
            Me.EditMenuBarItem.Name = "EditMenuBarItem"
            '
            'UndoBarItem
            '
            Me.UndoBarItem.Caption = "&Undo"
            Me.UndoBarItem.CategoryGuid = New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")
            Me.UndoBarItem.Id = 10
            Me.UndoBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.Z))
            Me.UndoBarItem.Name = "UndoBarItem"
            '
            'CutBarItem
            '
            Me.CutBarItem.Caption = "Cu&t"
            Me.CutBarItem.CategoryGuid = New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")
            Me.CutBarItem.Id = 12
            Me.CutBarItem.ImageIndex = 1
            Me.CutBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.X))
            Me.CutBarItem.Name = "CutBarItem"
            '
            'CopyBarItem
            '
            Me.CopyBarItem.Caption = "&Copy"
            Me.CopyBarItem.CategoryGuid = New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")
            Me.CopyBarItem.Id = 13
            Me.CopyBarItem.ImageIndex = 0
            Me.CopyBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.C))
            Me.CopyBarItem.Name = "CopyBarItem"
            '
            'PasteBarItem
            '
            Me.PasteBarItem.Caption = "&Paste"
            Me.PasteBarItem.CategoryGuid = New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")
            Me.PasteBarItem.Id = 14
            Me.PasteBarItem.ImageIndex = 3
            Me.PasteBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.V))
            Me.PasteBarItem.Name = "PasteBarItem"
            '
            'ClearBarItem
            '
            Me.ClearBarItem.Caption = "Cle&ar"
            Me.ClearBarItem.CategoryGuid = New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")
            Me.ClearBarItem.Id = 15
            Me.ClearBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut(System.Windows.Forms.Keys.Delete)
            Me.ClearBarItem.Name = "ClearBarItem"
            '
            'SelectBarItem
            '
            Me.SelectBarItem.Caption = "Select A&ll"
            Me.SelectBarItem.CategoryGuid = New System.Guid("ed905d80-839b-418f-82d0-346d168ddefc")
            Me.SelectBarItem.Id = 16
            Me.SelectBarItem.ItemShortcut = New DevExpress.XtraBars.BarShortcut((System.Windows.Forms.Keys.Control Or System.Windows.Forms.Keys.A))
            Me.SelectBarItem.Name = "SelectBarItem"
            '
            'ViewMenuBarItem
            '
            Me.ViewMenuBarItem.Caption = "&View"
            Me.ViewMenuBarItem.CategoryGuid = New System.Guid("e619b5ba-a0ac-4b7d-afa5-b84dd7eab0c0")
            Me.ViewMenuBarItem.Id = 9
            Me.ViewMenuBarItem.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.ToolbarsBarItem)})
            Me.ViewMenuBarItem.Name = "ViewMenuBarItem"
            '
            'ToolbarsBarItem
            '
            Me.ToolbarsBarItem.Caption = "&Toolbars"
            Me.ToolbarsBarItem.CategoryGuid = New System.Guid("dfb3a3c7-d8a9-42e3-a350-b3a781047071")
            Me.ToolbarsBarItem.Id = 18
            Me.ToolbarsBarItem.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.ToolbarsListBarItem)})
            Me.ToolbarsBarItem.Name = "ToolbarsBarItem"
            '
            'ToolbarsListBarItem
            '
            Me.ToolbarsListBarItem.Caption = "ToolbarListBarItem"
            Me.ToolbarsListBarItem.CategoryGuid = New System.Guid("dfb3a3c7-d8a9-42e3-a350-b3a781047071")
            Me.ToolbarsListBarItem.Id = 17
            Me.ToolbarsListBarItem.Name = "ToolbarsListBarItem"
            Me.ToolbarsListBarItem.Visibility = DevExpress.XtraBars.BarItemVisibility.OnlyInRuntime
            '
            'ActionsMenuBarItem
            '
            Me.ActionsMenuBarItem.Caption = "&Actions"
            Me.ActionsMenuBarItem.CategoryGuid = New System.Guid("e619b5ba-a0ac-4b7d-afa5-b84dd7eab0c0")
            Me.ActionsMenuBarItem.Id = 22
            Me.ActionsMenuBarItem.Name = "ActionsMenuBarItem"
            Me.ActionsMenuBarItem.Visibility = DevExpress.XtraBars.BarItemVisibility.Never
            '
            'HelpMenuBarItem
            '
            Me.HelpMenuBarItem.Caption = "&Help"
            Me.HelpMenuBarItem.CategoryGuid = New System.Guid("e619b5ba-a0ac-4b7d-afa5-b84dd7eab0c0")
            Me.HelpMenuBarItem.Id = 1
            Me.HelpMenuBarItem.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.AboutBarItem)})
            Me.HelpMenuBarItem.Name = "HelpMenuBarItem"
            '
            'AboutBarItem
            '
            Me.AboutBarItem.Caption = "&About Phoenix iTill System"
            Me.AboutBarItem.CategoryGuid = New System.Guid("99425a3d-94f2-4fca-a6f3-bcd1ea4dd9ef")
            Me.AboutBarItem.Id = 3
            Me.AboutBarItem.Name = "AboutBarItem"
            '
            'StandardBar
            '
            Me.StandardBar.BarName = "Standard"
            Me.StandardBar.DockCol = 0
            Me.StandardBar.DockRow = 1
            Me.StandardBar.DockStyle = DevExpress.XtraBars.BarDockStyle.Top
            Me.StandardBar.FloatLocation = New System.Drawing.Point(57, 150)
            Me.StandardBar.FloatSize = New System.Drawing.Size(76, 16)
            Me.StandardBar.LinksPersistInfo.AddRange(New DevExpress.XtraBars.LinkPersistInfo() {New DevExpress.XtraBars.LinkPersistInfo(Me.SaveCloseBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.SaveNewBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.CutBarItem, True), New DevExpress.XtraBars.LinkPersistInfo(Me.CopyBarItem), New DevExpress.XtraBars.LinkPersistInfo(Me.PasteBarItem)})
            Me.StandardBar.Text = "Standard"
            '
            'SaveCloseBarItem
            '
            Me.SaveCloseBarItem.Caption = "&Save And Close"
            Me.SaveCloseBarItem.CategoryGuid = New System.Guid("c3aced69-cd2a-4d58-8a80-811fa2689282")
            Me.SaveCloseBarItem.Id = 24
            Me.SaveCloseBarItem.Name = "SaveCloseBarItem"
            '
            'ChildFormBase
            '
            Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
            Me.ClientSize = New System.Drawing.Size(784, 430)
            Me.Controls.Add(Me.barDockControlLeft)
            Me.Controls.Add(Me.barDockControlRight)
            Me.Controls.Add(Me.barDockControlBottom)
            Me.Controls.Add(Me.barDockControlTop)
            Me.KeyPreview = True
            Me.MaximizeBox = False
            Me.MinimumSize = New System.Drawing.Size(530, 440)
            Me.Name = "ChildFormBase"
            Me.Text = "ChildFormBase"
            CType(Me.BarManager, System.ComponentModel.ISupportInitialize).EndInit()
            Me.ResumeLayout(False)

        End Sub

#End Region

#Region " Preferences & Configuration"
        Private Sub ChangeConfiguration()
            Try
                BarManager.BeginUpdate()
                BarManager.Images = AppSharedComponents.ToolImages
                BarManager.EndUpdate()
            Catch ex As Exception
                Trace.WriteIf(AppError, ex.ToString())
            End Try
        End Sub
#End Region

#Region " Form Event Handlers"
        Private Sub Form_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
            AddHandler PhoenixApplication.AppNotification, AddressOf Application_Notification
            AddHandler PhoenixApplication.AppLogingOut, AddressOf Application_Logout

            UpdateSecurityEditCommands()
            DataBind()
        End Sub

        Private Sub Form_Closed(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Closed
            RemoveHandler PhoenixApplication.AppNotification, AddressOf Application_Notification
        End Sub

        Private Sub Form_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs) Handles MyBase.KeyDown
            If e.KeyCode = Keys.Escape Then
                e.Handled = True
                Close()
            End If
        End Sub

        Private Sub Form_Closing(ByVal sender As Object, ByVal e As CancelEventArgs) Handles MyBase.Closing
            If mForceClose Then Return

            If DataSource.RowState = DataRowState.Detached Then
                AppRaiseEvent(Me, New PhoenixEventArgs(DataSource, PhoenixEventCode.BizRemoved))
                Return
            End If
            If IsDataUpdated() Then
                Activate()
                Dim result As DialogResult = AppMessageBox(String.Format(Res.SaveChanges, DocumentTitle), MsgBoxStyle.YesNoCancel Or MsgBoxStyle.Exclamation)
                Select Case result
                    Case DialogResult.Cancel
                        e.Cancel = True
                    Case DialogResult.No
                        RejectChanges()
                        AppRaiseEvent(Me, New PhoenixEventArgs(DataSource, PhoenixEventCode.BizUpdated))
                    Case DialogResult.Yes
                        If DataValidate() Then
                            Save()
                        Else
                            e.Cancel = True
                        End If
                End Select
            End If
        End Sub
#End Region

#Region " BarItem Event Handlers "
        Private Sub AboutBarItem_ItemClick(ByVal sender As System.Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles AboutBarItem.ItemClick
            AppAbout()
        End Sub

        Private Sub CustomizeBarItem_ItemClick(ByVal sender As System.Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs)
            BarManager.Customize()
        End Sub

        Private Sub CloseBarItem_ItemClick(ByVal sender As System.Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles CloseBarItem.ItemClick
            Close()
        End Sub

        Private Sub SaveBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles SaveBarItem.ItemClick
            Save()
        End Sub

        Private Sub SaveCloseBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles SaveCloseBarItem.ItemClick
            SaveAndClose()
        End Sub

        Private Sub SaveNewBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles SaveNewBarItem.ItemClick
            SaveAndNew()
        End Sub

        Private Sub DeleteBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles DeleteBarItem.ItemClick
            Delete()
        End Sub

#End Region

#Region " Edit commands hooks "
        Protected Sub HookControl(ByVal edit As TextEdit)
            AddHandler edit.Leave, AddressOf Edit_LostFocus
            AddHandler edit.KeyDown, AddressOf Edit_Changed
            AddHandler edit.KeyUp, AddressOf Edit_Changed
            AddHandler edit.MouseDown, AddressOf Edit_Changed
            AddHandler edit.MouseUp, AddressOf Edit_Changed
        End Sub
        Private Sub Edit_Changed(ByVal sender As Object, ByVal args As MouseEventArgs)
            UpdateEditCommands(CType(sender, TextEdit))
        End Sub

        Private Sub Edit_Changed(ByVal sender As Object, ByVal args As KeyEventArgs)
            UpdateEditCommands(CType(sender, TextEdit))
        End Sub

        Private Sub Edit_LostFocus(ByVal sender As Object, ByVal args As EventArgs)
            UndoBarItem.Enabled = False
            CopyBarItem.Enabled = False
            PasteBarItem.Enabled = False
            SelectBarItem.Enabled = False
            CutBarItem.Enabled = False
            ClearBarItem.Enabled = False
        End Sub

        Private Sub HooksLoad(ByVal sender As Object, ByVal args As EventArgs) Handles MyBase.Load
            UndoBarItem.Enabled = False
            CopyBarItem.Enabled = False
            PasteBarItem.Enabled = False
            SelectBarItem.Enabled = False
            CutBarItem.Enabled = False
            ClearBarItem.Enabled = False
            HookControlRecursive(Me)
        End Sub

        Private Sub HookControlRecursive(ByVal container As Control)
            For Each control As Control In container.Controls
                If TypeOf control Is TextEdit Then
                    HookControl(control)
                Else
                    HookControlRecursive(control)
                End If
            Next
        End Sub

        Private Sub UpdateEditCommands(ByVal edit As TextEdit)
            SelectBarItem.Enabled = IIf(edit.Text.Length = 0, False, True)
            ClearBarItem.Enabled = IIf(edit.SelectionLength = 0, False, True)
            CutBarItem.Enabled = IIf(edit.SelectionLength = 0, False, True)
            CopyBarItem.Enabled = IIf(edit.SelectionLength = 0, False, True)
            UndoBarItem.Enabled = edit.CanUndo
            Try
                If Clipboard.GetDataObject().GetDataPresent(DataFormats.Text) Then
                    If CType(Clipboard.GetDataObject().GetData(DataFormats.Text), String).Length > 0 Then
                        PasteBarItem.Enabled = True
                    Else
                        PasteBarItem.Enabled = False
                    End If
                Else
                    PasteBarItem.Enabled = False
                End If
            Catch
                PasteBarItem.Enabled = False
            End Try
        End Sub
#End Region

#Region " Life-cycle Routines "

        Public Function Save() As Boolean
            SelectNextControl(ActiveControl, True, False, False, True)
            SelectNextControl(ActiveControl, False, False, False, True)



            BindingContext(DataSource).EndCurrentEdit()


            If DataValidate() Then
                Try
                    DoSaving()
                Catch ex As CancelOperationException
                    Return False
                    'TTP #334
                    'Catch ex As DataAccessException
                    '    If Not ex.InnerException Is Nothing Then
                    '        If Not TypeOf ex.InnerException Is CancelOperationException Then
                    '            AppMessageBox(ex.Message, MsgBoxStyle.Critical)
                    '        End If
                    '    Else
                    '        AppMessageBox(ex.Message, MsgBoxStyle.Critical)
                    '    End If
                    '    Return False
                Catch ex As Exception
                    'AppMessageBox(ex.Message, MsgBoxStyle.Critical)
                    'Return False
                    Throw
                End Try

                DataBind()
                Return True
            End If

            Return False
        End Function

        Public Sub SaveAndClose()
            If Save() Then
                ForceClose()
            End If
        End Sub

        Public Sub SaveAndNew()
            If Save() Then
                NewRow()
                DataBind()
            End If
        End Sub

        Private Sub DataBind()
            ClearBinding(Me)

            Text = Title

            DoBinding()
        End Sub

        Private Sub Delete()
            OnDelete()
            Close()
        End Sub

        Protected Sub ClearBinding(ByVal container As Control)
            container.DataBindings.Clear()
            For Each control As Control In container.Controls
                ClearBinding(control)
            Next
        End Sub

        Protected Overridable Sub DoSaving()
        End Sub

        Protected Overridable Sub DoBinding()
        End Sub

        Private mCreationParam As Object = Nothing
        Public Property CreationParameter() As Object
            Get
                Return mCreationParam
            End Get
            Set(ByVal Value As Object)
                mCreationParam = Value
            End Set
        End Property

        Private Sub NewRow()
            DataSource = CreateNewRow()
            CreationParameter = Nothing
        End Sub

        'TODO:  MustInherits
        Protected Overridable Function CreateNewRow() As DataRow
            Return Nothing
        End Function

        Public Event Deleted(ByVal sender As Object, ByVal e As EventArgs)
        Protected Overridable Sub OnDelete()
            RaiseEvent Deleted(Me, EventArgs.Empty)
        End Sub

        Protected Overridable Sub UpdateSecurityEditCommands()

        End Sub

        Private Sub Application_Logout(ByVal sender As Object, ByVal e As CancelEventArgs)
            If e.Cancel Then Return

            Close()

            If Visible Then
                e.Cancel = True
            End If
        End Sub

        Private Sub Application_Notification(ByVal sender As Object, ByVal e As PhoenixEventArgs)
            If DataSource Is Nothing Then Return
            If sender Is Me Then Return

            Select Case e.EventCode
                Case PhoenixEventCode.ConfigurationChanged
                    ChangeConfiguration()
            End Select

            OnApplicationEvent(sender, e)
        End Sub

        Protected Overridable Sub OnApplicationEvent(ByVal sender As Object, ByVal e As PhoenixEventArgs)

        End Sub

        Private mForceClose As Boolean = False
        Protected Sub ForceClose()
            mForceClose = True
            Close()
        End Sub
#End Region

#Region " Edit commands handlers "
        Private Sub ClearBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles ClearBarItem.ItemClick
            If TypeOf Me.ActiveControl.Parent Is TextEdit Then
                CType(Me.ActiveControl.Parent, TextEdit).SelectedText = ""
            End If
        End Sub


        Private Sub CopyBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles CopyBarItem.ItemClick
            If TypeOf Me.ActiveControl.Parent Is TextEdit Then
                CType(Me.ActiveControl.Parent, TextEdit).Copy()
            End If
        End Sub


        Private Sub CutBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles CutBarItem.ItemClick
            If TypeOf Me.ActiveControl.Parent Is TextEdit Then
                CType(Me.ActiveControl.Parent, TextEdit).Cut()
            End If
        End Sub


        Private Sub PasteBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles PasteBarItem.ItemClick
            If TypeOf Me.ActiveControl.Parent Is TextEdit Then
                CType(Me.ActiveControl.Parent, TextEdit).Paste()
            End If
        End Sub


        Private Sub SelectBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles SelectBarItem.ItemClick
            If TypeOf Me.ActiveControl.Parent Is TextEdit Then
                CType(Me.ActiveControl.Parent, TextEdit).SelectAll()
                UpdateEditCommands(Me.ActiveControl.Parent)
            End If
        End Sub


        Private Sub UndoBarItem_ItemClick(ByVal sender As Object, ByVal e As DevExpress.XtraBars.ItemClickEventArgs) Handles UndoBarItem.ItemClick
            If TypeOf Me.ActiveControl.Parent Is TextEdit Then
                CType(Me.ActiveControl.Parent, TextEdit).Undo()
            End If
        End Sub
#End Region

#Region " Data Validation "
        Private mDataValidation As Hashtable = New Hashtable

        Protected Sub AddControlToValidate(ByVal c As Control, ByVal handler As CancelEventHandler)
            If c Is Nothing Then Return
            If handler Is Nothing Then Return
            If mDataValidation.Contains(c) Then Return

            AddHandler c.Leave, AddressOf ControlValidation_Leave
            mDataValidation.Add(c, handler)
        End Sub

        Public Overridable Function DataValidate() As Boolean
            Dim cea As CancelEventArgs = New CancelEventArgs(False)
            For Each key As Control In mDataValidation.Keys
                Dim handler As CancelEventHandler = mDataValidation(key)
                handler(key, cea)
            Next
            Return Not cea.Cancel
        End Function

        Private Sub ControlValidation_Leave(ByVal Sender As Object, ByVal e As EventArgs)
            Dim handler As CancelEventHandler = mDataValidation(Sender)
            If Not handler Is Nothing Then
                handler(Sender, New CancelEventArgs(False))
            End If
        End Sub

#End Region

#Region " Data Exchange "
        Private mSuspendDataChangedEventCount As Integer = 0

        Protected Sub SuspendChangedEvents()
            mSuspendDataChangedEventCount = mSuspendDataChangedEventCount + 1
        End Sub

        Protected Sub ResumeChangedEvents()
            mSuspendDataChangedEventCount = mSuspendDataChangedEventCount - 1
            If (mSuspendDataChangedEventCount < 0) Then mSuspendDataChangedEventCount = 0
        End Sub

        Protected Property SuspendDataChangedEvents()
            Get
                If mSuspendDataChangedEventCount = 0 Then
                    Return False
                Else
                    Return True
                End If
            End Get
            Set(ByVal Value)
                If Value Then
                    SuspendChangedEvents()
                Else
                    ResumeChangedEvents()
                End If
            End Set
        End Property
#End Region

#Region " Resize Routines "
        Private Sub ChildFormBase_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
            If Width > MaxWidth Then
                Width = MaxWidth
            End If

            If Height > MaxHeight Then
                Height = MaxHeight
            End If
        End Sub

        Public ReadOnly Property MaxWidth() As Integer
            Get
                If mMaxWidth = 0 Then
                    mMaxWidth = Math.Round(Screen.PrimaryScreen.WorkingArea.Width * 0.75)
                End If
                Return mMaxWidth
            End Get
        End Property

        Public ReadOnly Property MaxHeight() As Integer
            Get
                If mMaxHeigh = 0 Then
                    mMaxHeigh = Math.Round(Screen.PrimaryScreen.WorkingArea.Height * 0.75)
                End If
                Return mMaxHeigh
            End Get
        End Property

        Private Shared mMaxWidth As Integer = 0
        Private Shared mMaxHeigh As Integer = 0
#End Region
    End Class

End Namespace
